## Table 2 in paper
opttable <- function(mp, op, sigma2) {
    fn <- function(mp, opt, oc) {
        fn  <- if (opt)
                   ATEHonest::ATTOptEstimate
               else
                   ATEHonest::ATTMatchEstimate
        fn(path=mp, sigma2=sigma2, C=1, opt.criterion=oc)$e
    }

    t1 <- data.frame(rbind(fn(op, TRUE, "RMSE"), fn(op, TRUE, "FLCI")))
    t2 <- data.frame(rbind(fn(mp, FALSE, "RMSE"), fn(mp, FALSE, "FLCI")))
    t1$M <- NA
    t1$eff <- 100*c(t1$hl[1]/t1$hl[2], NA)
    t2$delta <- t2$omega <- NA
    t2$eff <- 100*c(t1$rmse[1]/t2$rmse[1], t1$hl[2]/t2$hl[2])
    topt <- rbind(t1, t2)
    topt$cv <- topt$rhl/topt$rsd

    ## CIs
    topt$cilc <- topt$att-topt$rhl
    topt$ciuc <- topt$att+topt$rhl
    topt$cilp <- topt$att-topt$uhl
    topt$ciup <- topt$att+topt$uhl

    rownames(topt) <- c("Opt: RMSE", "Opt: FLCI", "Matching: RMSE",
                        "Matching: FLCI")
    topt <- topt[, c("M", "delta", "att", "maxbias", "rsd", "cv", "cilc",
                     "ciuc", "usd", "cilp", "ciup", "lindw", "eff")]
    colnames(topt) <- c("$M$", "$\\delta$", "Estimate", "Max. Bias", "SE", "cv",
                        "CI lo", "CI hi", "SE (PATE)", "CI lo (PATE)",
                        "CI hi (PATE)", "Lindeberg wgt", "Rel. efficiency")
    topt
}


#' Estimator for a range of Cs
#' @param opt Matching or Optimal estimator?
pathestimate <- function(Cs, mp, sigma2, drop=TRUE, opt=TRUE) {
    fn  <- if (opt) ATEHonest::ATTOptEstimate else ATEHonest::ATTMatchEstimate

    f <- function(oc) data.frame(t(sapply(Cs, function(C)
        unlist(fn(path=mp, sigma2=sigma2, C=C, opt.criterion=oc)$e))))
    flci <- f("FLCI")
    rmse <- f("RMSE")

    ## Drop Cs for which we found optimum at a boundary
    bd <- if (drop && opt) {
              md <- max(mp$ep$delta)*Cs/sqrt(mean(sigma2))
              flci$delta==md | rmse$delta==md
          } else if (drop && !opt)  {
              Mm <- max(mp$ep$M)
              flci$M==Mm | rmse$M== Mm
          } else {
              rep(FALSE, nrow(flci))
          }

    list(flci=flci[!bd, ], rmse=rmse[!bd, ])
}

PlotLind <- function(mp, op, sigma2) {
    mpu <- ATEHonest:::UpdatePath(mp$ep, mp$K, Cratio=1, sigma2,
                                  alpha=0.05, beta=0.8)
    opu <- ATEHonest:::UpdatePath(op$ep, op$K, Cratio=1, sigma2,
                                  alpha=0.05, beta=0.8)
    mpu$type <- "Matching estimator"
    opu$type <- "Feasible estimator"
    mpu$x <- "M"
    opu$x <- "d"
    opu <- opu[-1, ]
    opu <- opu[opu$delta <= 10, ]
    names(mpu)[3] <- "delta"
    df <- rbind(mpu[, c("type", "delta", "lindw", "x")],
                opu[, c("type", "delta", "lindw", "x")])

    f1 <- ggplot2::qplot(x=delta, y=lindw, data=df, geom="line") +
        ggplot2::facet_wrap(. ~ x, scales="free_x", strip.position = "bottom",
                            labeller = ggplot2::as_labeller(c(M="$M$",
                                                              d="$\\delta$"))) +
        ggplot2::xlab(NULL) + ggplot2::ylab("Max. Lindeberg weight") +
        ggplot2::theme(strip.background = ggplot2::element_blank(),
                       strip.placement = "outside") +
          ggplot2::geom_hline(yintercept=0.075) +
          ggplot2::scale_y_continuous(breaks = c(0, 0.075, 0.15))
    list(f=f1, d=df)
}


#' Plot estimator for a range of Cs
PlotOptpath <- function(oe) {
    ## ggplot orders them alphabetically
    var <- c("s1", "s2", "t1", "t2", "u")

    d <- data.frame(C=rep(oe$flci$C, 5),
                    y=c(oe$flci$att-oe$flci$uhl,
                        oe$flci$att+oe$flci$uhl,
                        oe$flci$att-oe$flci$rhl,
                        oe$flci$att+oe$flci$rhl, oe$rmse$att),
                    var=rep(var, each=nrow(oe$flci)),
                    lbl=rep(var, each=nrow(oe$flci)))
    p1 <- ggplot2::ggplot(oe$rmse[oe$rmse$C<=2, ]) +
        ggplot2::geom_ribbon(ggplot2::aes(x=C, ymin=att-maxbias,
                                          ymax=att+maxbias), alpha=0.1) +
        ggplot2::geom_line(data=d[d$C<=2, ],
                  ggplot2::aes(x=C, y=y, group=var, linetype=lbl)) +
        ggplot2::scale_linetype_manual(values=c("dotted", "dotted", "solid",
                                                "solid", "dashed"))+
        ggplot2::theme(legend.position="none") +
        ggplot2::labs(x="$C$", y="Effect of training program")
    lbl <- d[abs(d$C-1.5)<1e-10, ]
    lbl$label <- c("", "95\\% PATT CI", "", "95\\% CATT CI", "Estimate")
    p1 <- p1+
        ggrepel::geom_text_repel(data = lbl,
                                 ggplot2::aes(x=C, y=y, label=label), size=3)

    p1
}


#' Efficiency of matching estimator with M=1 relative to optimal estimator
#' @param oe Output of pathestimate for optimal esitmator
#' @param mp Path for matching estimator
MatchEfficiency <- function(oe, mp, sigma2, mineff=95.0) {
    ## Path for matching with M=1
    mp1 <- mp
    mp1$ep <- mp$ep[1, ]
    mp1$K <- mp$K[1, , drop=FALSE]
    me1 <- pathestimate(oe$rmse$C, mp1, sigma2, drop=FALSE, opt=FALSE)
    d <- data.frame(C=rep(oe$rmse$C, 2),
                    type=rep(c("RMSE", "FLCI"), each=length(oe$rmse$C)),
                    eff=100*c(oe$rmse$rmse/me1$rmse$rmse,
                              oe$flci$hl/me1$flci$hl))
    idx <- c(which.max(d$eff>mineff & d$type=="RMSE"),
             which.max(d$eff>mineff & d$type=="FLCI"))
    p <- ggplot2::qplot(data=d, x=C, y=eff, linetype=type, geom="line") +
        ggplot2::geom_hline(yintercept = mineff) +
        ggplot2::labs(y="Relative Efficiency (\\%)", x="$C$")
    list(p=p, effC1=d[d$C==1, ], effCmin=d[idx, ], d=d)
}


#' Standard errors for difference in means estimator
RCTvariance <- function(y, d, sigma2, D0) {
    mpI <- ATEHonest::ATTMatchPath(y, d, D0, M=Inf)
    me <- ATEHonest::ATTMatchEstimate(mpI, sigma2=sigma2, C=1)

    n1 <- sum(d)
    n0 <- sum(1-d)
    ## Conditional variance
    se <- c(me$e["rsd"], me$e["sd"])
    ## Add CATT variance and ATE variance
    se <- c(se^2, se^2+(var(y[d==0])-mean(sigma2[d==0])) /
                     (n1*n0/(n1+n0)),
            var(y[d==1])/n1+var(y[d==0])/n0,
            var(y[d==1])*(n1-1)/n1^2+var(y[d==0])*(n0-1)/n0^2,
            (var(y[d==1])*(n1-1)+var(y[d==0])*(n0-1)) /
            (n1+n0-2) * (1/n1+1/n0)
            )
    names(se) <- c("Cond rob", "Cond hom", "CATT rob", "CATT hom", "PATE HC2",
                   "PATE EHW", "PATE hom")
    c(me$e["att"], me$e["maxbias"], sqrt(se))
}
